/*
 * Sudokuki - essential sudoku game
 * Copyright (C) 2007-2011 Sylvain Vedrenne
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.jankenpoi.sudokuki.model;

import java.util.ArrayList;
import java.util.List;

import net.jankenpoi.sudokuki.SudokuGrid;
import net.jankenpoi.sudokuki.suexg.SuexgProxy;
import net.jankenpoi.sudokuki.view.GridListener;

/**
 * (MVC) model for a sudoku grid
 * 
 * @author sylvain
 * 
 */
public class GridModel {

	public static final short MASK_VALUES = 0x000F; // //0000.0000-0000.1111
	public static final short FLAG_READ_ONLY = 0x0010; // //0000.0000-0001.0000
	public static final short FLAG_CELL_MEMO_1 = 0x0040; // 0000.0000-0100.0000
	public static final short FLAG_CELL_MEMO_2 = 0x0080; // 0000.0000-1000.0000
	public static final short FLAG_CELL_MEMO_3 = 0x0100; // 0000.0001-0000.0000
	public static final short FLAG_CELL_MEMO_4 = 0x0200; // 0000.0010-0000.0000
	public static final short FLAG_CELL_MEMO_5 = 0x0400; // 0000.0100-0000.0000
	public static final short FLAG_CELL_MEMO_6 = 0x0800; // 0000.1000-0000.0000
	public static final short FLAG_CELL_MEMO_7 = 0x1000; // 0001.0000-0000.0000
	public static final short FLAG_CELL_MEMO_8 = 0x2000; // 0010.0000-0000.0000
	public static final short FLAG_CELL_MEMO_9 = 0x4000; // 0100.0000-0000.0000
	public static final short FLAG_CELL_MEMO_ALL = 0x7FC0; // 0111.1111-1100.0000

	private List<GridListener> listeners = new ArrayList<GridListener>();

	/**
	 * 
	 * Values
	 */
	private short[] cellInfos = new short[91];

	public GridModel() {
		SudokuGrid grid = SuexgProxy.getInstance().generateGrid();

		for (int i = 1; i < cellInfos.length; i++) {
			short value = (short) grid.getValueAt(i);
			cellInfos[i] = value;
			if (1 <= value && value <= 9) {
				cellInfos[i] |= FLAG_READ_ONLY;
			}
		}
	}

	public void addGridListener(GridListener view) {
		listeners.add(view);
	}

	public short getCellInfosAt(int li, int co) {
		return cellInfos[9 * li + co];
	}

	public void setAllCellMemos() {
		//FIXME: TODO: ..................
		
		// parcourir tous les carres
		// - pour chaque carre, cribler les memos
		// -- pour chaque cellule, cribler les memos
		//
		// parcourir toutes les lignes
		// - pour chaque ligne, cribler les memos
		// -- pour chaque cellule, cribler les memos
		//
		// parcourir toutes les colonnes
		// - pour chaque colonne, cribler les memos
		// -- pour chaque cellule, cribler les memos
		/*
		0 3 6
		3
		6
		*/
		
		for (int X = 0; X < 9; X+=3) { // left pos of a square
			for (int Y = 0; Y < 9; Y+=3) { // top pos of a square
				short possibleValuesMask = 0;
				for (int x = 0; x < 3; x++) {
					for (int y = 0; y < 3; y++) {
						int co = X + x;
						int li = Y + y;
						
					}
				}
				for (int x = 0; x < 3; x++) {
					for (int y = 0; y < 3; y++) {
						int co = X + x;
						int li = Y + y;
						
					}
				}
			}
		}
		
	}
	
	public void setCellMemos(int li, int co, byte[] memos) {
		for (int i = 0; i < memos.length; i++) {
			setCellMemo(li, co, memos[i]);
		}
		fireGridChanged(new GridChangedEvent(this, li, co, cellInfos[9 * li
				+ co]));
	}

	private short getMemoFlag(byte memo) {
		switch (memo) {
		case 1:
			return FLAG_CELL_MEMO_1;
		case 2:
			return FLAG_CELL_MEMO_2;
		case 3:
			return FLAG_CELL_MEMO_3;
		case 4:
			return FLAG_CELL_MEMO_4;
		case 5:
			return FLAG_CELL_MEMO_5;
		case 6:
			return FLAG_CELL_MEMO_6;
		case 7:
			return FLAG_CELL_MEMO_7;
		case 8:
			return FLAG_CELL_MEMO_8;
		case 9:
			return FLAG_CELL_MEMO_9;
		default:
			throw new IllegalArgumentException();
		}
	}
	
	private void setCellMemo(int li, int co, byte memo) {
			cellInfos[9 * li + co] |= getMemoFlag(memo);
	}

	public void clearCellMemos(int li, int co) {
		cellInfos[9 * li + co] &= ~FLAG_CELL_MEMO_ALL;
	}

	public boolean isCellValueSet(int li, int co, Byte value) {
		return (cellInfos[9 * li + co] & value.byteValue()) != 0;
	}

	/**
	 * For use for initial values in the grid, and when doing a custom grid, for
	 * instance...
	 * 
	 * @param li
	 * @param co
	 */
	public void setCellReadOnly(int li, int co) {
		cellInfos[9 * li + co] |= FLAG_READ_ONLY;
	}

	public void setCellValue(int li, int co, int value) {
		System.out.println("GridModel.setCellValue() li:" + li + " co:" + co
				+ " value:" + value);
		System.out.println("GridModel.setCellValue() cellInfos[9*li+co]:"
				+ cellInfos[9 * li + co]);
		cellInfos[9 * li + co] &= ~MASK_VALUES;
		cellInfos[9 * li + co] |= value;
		clearCellMemos(li, co);
		System.out.println("GridModel.setCellValue() cellInfos[9*li+co]:"
				+ cellInfos[9 * li + co] + " (after)...");
		fireGridChanged(new GridChangedEvent(this, li, co, cellInfos[9 * li
				+ co]));
	}

	public void fireGridChanged(GridChangedEvent event) {

		for (GridListener listener : listeners) {
			listener.gridChanged(event);
		}
	}

	public boolean isCellMemoSet(int li, int co, byte memo) {
			return (cellInfos[9 * li + co] & getMemoFlag(memo)) != 0;
	}

	public boolean isCellReadOnly(int li, int co) {
		// System.out.println("GridView.isCellReadOnly() cellInfos[9 * li + co]:"+cellInfos[9
		// * li + co]);
		return (cellInfos[9 * li + co] & GridModel.FLAG_READ_ONLY) != 0;
	}

	public byte getValueAt(int li, int co) {
		byte value = (byte) (cellInfos[9 * li + co] & GridModel.MASK_VALUES);
		return value;
	}

}
