/*
 * Sudokuki - essential sudoku game
 * Copyright (C) 2007-2011 Sylvain Vedrenne
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.jankenpoi.sudokuki.suexg;

import java.util.Random;

import net.jankenpoi.sudokuki.SudokuGrid;

public class SuexgProxy {

	static {
		String javaLibraryPath = System.getProperty("java.library.path");
		System.out.println("SuexgProxy.enclosing_method() java.library.path:"
				+ javaLibraryPath);
		System.loadLibrary("suexg_proxy");
		System.out.println("SuexgProxy.enclosing_method() after call to loadLibrary()");
	}

	private static SuexgProxy proxy = new SuexgProxy();
	
	private SuexgProxy() {
	}
	
	public static SuexgProxy getInstance() {
		return proxy;
	}
	
	public static void main(String[] args) {
		System.out.println("SuexgProxy.main()");
		Random rand = new Random(System.currentTimeMillis());

		int[] grid = new int[81];
		int[] gridAndClues = new int[81];
		int[] rating = new int[] { -1 };
		System.out.println("\nSuexgProxy.main() APRES");
		int seed = rand.nextInt();
		int status = proxy.generateSuexgGrid(seed, grid, rating,
				gridAndClues);
		System.out.println("rating:" + rating[0] + "\nstatus:" + status);
		System.out.println("\n*** generated grid ***");
		printGrid(grid);
		System.out.println("\n*** generated grid with clues ***");
		printGrid(gridAndClues);
		
//		int[] outGrid = new int[81];
//		proxy.solveCustomGrid(grid, outGrid);
//		System.out.println("\n*** calculated solution (outGrid) ***");
//		printGrid(outGrid);
		
		System.out.println("SuexgProxy.enclosing_method()");
	}

	public SudokuGrid generateGrid() {
		Random rand = new Random(System.currentTimeMillis());

		int[] grid = new int[81];
		int[] gridAndClues = new int[81];
		int[] rating = new int[] { -1 };
		System.out.println("\nSuexgProxy.main() APRES");
		int seed = rand.nextInt();
		int status = proxy.generateSuexgGrid(seed, grid, rating,
				gridAndClues);
		System.out.println("rating:" + rating[0] + "\nstatus:" + status);
		System.out.println("\n*** generated grid ***");
		printGrid(grid);
		System.out.println("\n*** generated grid with clues ***");
		printGrid(gridAndClues);
		
		SudokuGrid sudoku = new SudokuGrid(grid);
		return sudoku;
	}
	
	native int generateSuexgGrid(int inSeed, int[] outGrid, int[] outRating,
			int[] outSolution);

	native int solveCustomGrid(int[] inGrid, int[] outGrid);

	private static void printGrid(int[] tab) {
		for (int i = 0; i < tab.length; i++) {
			if (i % 3 == 0)
				System.out.print(" ");
			if (i % 9 == 0)
				System.out.println();
			if (i % 27 == 0)
				System.out.println();
			System.out.print("" + (tab[i] == 0 ? "-" : tab[i]));
		}
		System.out.println();
	}
}
