/*
 * Sudokuki - essential sudoku game
 * Copyright (C) 2007-2011 Sylvain Vedrenne
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.jankenpoi.sudokuki.ui.swing;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridLayout;
import java.util.HashSet;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.WindowConstants;

/*
 * InputDialog.java
 *
 * Created on Mar 25, 2011, 5:50:15 PM
 */

/**
 * 
 * @author svedrenne
 */
@SuppressWarnings("serial")
public class SelectNotesDialog extends JDialog {

	private JButton btnClear;
	private JCheckBox[] ckb = new JCheckBox[10];
	private JButton btnConfirm;
	private JPanel panelClear = new JPanel(new GridLayout());
	private JPanel panel789 = new JPanel(new GridLayout());
	private JPanel panel456 = new JPanel(new GridLayout());
	private JPanel panel123 = new JPanel(new GridLayout());
	private JPanel panelConfirm = new JPanel(new GridLayout());
	private HashSet<Byte> notes = new HashSet<Byte>();

	private Font BOLD_FONT = new Font("Serif", Font.BOLD, 18);
	private Font NORMAL_FONT = new Font("Serif", Font.PLAIN, 18);

	public SelectNotesDialog(Frame parent, Byte[] previousNotes) {
		super(parent, true);
		initComponents(previousNotes);
		setResizable(false);
	}

	private void configureCheckBox(JCheckBox btn, String text, final int value) {
		// System.out.println("InputDialog.configureButton() btn:"+btn+" text:"+text+" value:"+value);

//		btn.setBorderPainted(false);
		btn.setFont(NORMAL_FONT);

//		btn.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		btn.setToolTipText(text);
		btn.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
		btn.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				buttonClicked(value);
			}
		});
	}

	private void initComponents(Byte[] previousNotes) {

		if (previousNotes != null) {
			for (int i = 0; i < previousNotes.length; i++) {
				notes.add(previousNotes[i]);
			}
		}
		btnClear = new JButton();
		for (int i = 1; i <= 9; i++) {
			ckb[i] = new JCheckBox(String.valueOf(i));
		}
		btnConfirm = new JButton();

		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		btnClear.setText("Clear notes");
		btnClear.setEnabled(true);
		btnClear.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				clearClicked();
			}
			private void clearClicked() {
				for (byte i=1; i<=9; i++) {
					notes.remove(i);
					ckb[i].setFont(NORMAL_FONT);
					ckb[i].setSelected(false);
				}
			}
		});
		configureCheckBox(ckb[1], "1", 1);
		configureCheckBox(ckb[2], "2", 2);
		configureCheckBox(ckb[3], "3", 3);
		configureCheckBox(ckb[4], "4", 4);
		configureCheckBox(ckb[5], "5", 5);
		configureCheckBox(ckb[6], "6", 6);
		configureCheckBox(ckb[7], "7", 7);
		configureCheckBox(ckb[8], "8", 8);
		configureCheckBox(ckb[9], "9", 9);
		btnConfirm.setText("Ok");
		btnConfirm.setEnabled(true);
		btnConfirm.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				confirmClicked();
			}

			private void confirmClicked() {
				dispose();
			}
		});

		Iterator<Byte> it = notes.iterator();
		while (it.hasNext()) {
			JToggleButton button = ckb[it.next()];
			button.setSelected(true);
			button.setFont(BOLD_FONT);
		}
		

		Container pane = getContentPane();
		GridLayout btnLayout = new GridLayout(5, 1);
		pane.setLayout(btnLayout);
		pane.add(panelClear);
		pane.add(panel123);
		pane.add(panel456);
		pane.add(panel789);
		pane.add(panelConfirm);

		panelClear.add(btnClear);
		panel789.add(ckb[7], BorderLayout.LINE_START);
		panel789.add(ckb[8], BorderLayout.CENTER);
		panel789.add(ckb[9], BorderLayout.LINE_END);
		panel456.add(ckb[4], BorderLayout.LINE_START);
		panel456.add(ckb[5], BorderLayout.CENTER);
		panel456.add(ckb[6], BorderLayout.LINE_END);
		panel123.add(ckb[1], BorderLayout.LINE_START);
		panel123.add(ckb[2], BorderLayout.CENTER);
		panel123.add(ckb[3], BorderLayout.LINE_END);
		panelConfirm.add(btnConfirm);

		pack();
	}

	private void buttonClicked(int button) {
		System.out.println("InputDialog.buttonClicked() button:" + button);
		if (ckb[button].isSelected()) {
			notes.add((byte) button);
			ckb[button].setFont(BOLD_FONT);
		} else {
			notes.remove((byte) button);
			ckb[button].setFont(NORMAL_FONT);
		}
	}

	public byte[] getSelectedNotes() {
		byte[] notesArray = new byte[notes.size()];

		Iterator<Byte> it = notes.iterator();
		int i = 0;
		while (it.hasNext()) {
			notesArray[i] = it.next();
			i++;
		}
		return notesArray;
	}

}
