/*
 * Sudokuki - essential sudoku game
 * Copyright (C) 2007-2011 Sylvain Vedrenne
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.jankenpoi.sudokuki.ui.swing;

import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JFrame;
import javax.swing.JToolBar;

import net.jankenpoi.sudokuki.model.GridChangedEvent;
import net.jankenpoi.sudokuki.model.GridModel;
import net.jankenpoi.sudokuki.view.GridView;

public class SwingView extends GridView {

	private SwingGrid grid;
	private JFrame frame;

	public SwingView(GridModel model) {
		super(model);
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				frame = new JFrame("Sudokuki");
				frame.setLayout(new BorderLayout());
				grid = new SwingGrid(SwingView.this, frame);
				frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				JToolBar toolbar = new ToolBar(grid.getPrintAction(), grid
						.getCheckUpdateAction(),
						grid.getOpenUpdateSiteAction(), grid.getAboutAction());
				frame.getContentPane().add(toolbar, BorderLayout.PAGE_START);
				frame.getContentPane().add(grid, BorderLayout.CENTER);
				// frame.setLayout(new FlowLayout());

				frame.setJMenuBar(new MenuBar(grid.getPrintAction(), grid
						.getPrint4Action(), grid.getCheckUpdateAction(), grid
						.getOpenUpdateSiteAction(), grid.getAboutAction()));

				frame.pack();
				frame.setLocationRelativeTo(null);
			}
		});
	}

	@Override
	public void display() {
		System.out.println("SwingView.display()");
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				frame.setVisible(true);
			}
		});
		System.out.println("SwingView.display()/");
	}

	public void gridChanged(GridChangedEvent event) {
		super.gridChanged(event);
		System.out.println("SwingView.gridChanged()");
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				grid.repaint();
			}
		});
	}

	@Override
	public void close() {
	}

}
